// Copyright Epic Games, Inc. All Rights Reserved.

#include "hstherac25.h"
#include "HAL/PlatformProcess.h"
#include "Interfaces/IPluginManager.h"
#include "Misc/MessageDialog.h"
#include "Misc/Paths.h"
#include "Modules/ModuleManager.h"
#include "Therac.h"

#define LOCTEXT_NAMESPACE "Fhstherac25Module"

void Fhstherac25Module::StartupModule() {
  // This code will execute after your module is loaded into memory; the exact
  // timing is specified in the .uplugin file per-module

  // Get the base directory of this plugin
  FString BaseDir =
      IPluginManager::Get().FindPlugin("hstherac25")->GetBaseDir();

  // Add on the relative location of the third party dll and load it
  FString LibraryPath;
#if PLATFORM_WINDOWS
  LibraryPath = FPaths::Combine(*BaseDir, TEXT("Source/hstherac-hs/HSdll.dll"));

#elif PLATFORM_MAC
  LibraryPath = FPaths::Combine(
      *BaseDir,
      TEXT("Source/ThirdParty/hstherac25Library/Mac/Release/"
           "libExampleLibrary.dylib")
  );
#elif PLATFORM_LINUX
  LibraryPath = FPaths::Combine(
      *BaseDir,
      TEXT("Binaries/ThirdParty/hstherac25Library/Linux/"
           "x86_64-unknown-linux-gnu/libExampleLibrary.so")
  );
#endif // PLATFORM_WINDOWS

  simulatorLibraryHandle = !LibraryPath.IsEmpty()
                               ? FPlatformProcess::GetDllHandle(*LibraryPath)
                               : nullptr;

  if (simulatorLibraryHandle) {
    // Call the test function in the third party library that opens a message
    // box
    //    ExampleLibraryFunction();
    //    HsStablePtr wc = start_machine();
  } else {
    FMessageDialog::Open(
        EAppMsgType::Ok,
        LOCTEXT(
            "ThirdPartyLibraryError",
            "Failed to load example third party library"
        )
    );
  }
}

void Fhstherac25Module::ShutdownModule() {
  // This function may be called during shutdown to clean up your module.  For
  // modules that support dynamic reloading, we call this function before
  // unloading the module.

  // Free the dll handle
  FPlatformProcess::FreeDllHandle(simulatorLibraryHandle);
  simulatorLibraryHandle = nullptr;
}

#undef LOCTEXT_NAMESPACE

IMPLEMENT_MODULE(Fhstherac25Module, hstherac25)
